;################################################################################
;#										#
;# chipbasic - single chip basic computer with ATmega32				#
;# menu functions								#
;# copyright (c) 2006/2007 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.equ	menu_max = 7
;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
menu_main:	ldi	XL,1
		libmio_chpump			;enable chargepump for serial
		sts	libmio_channel,byte0	;output to screen
		rcall	menu_maxbox
		call	mem_cpadr		;get start address of prg
		clr	ereg		
		clr	ctrl
		libmio_thistext
		.db 2,5,0
		rcall	menu_itext1		;version
		libmio_thistext
		.db 3,4,0x9c,0x9c,0x9c,0xac,0xac,0xac,0xbc,0xbc,0xbc,0xcc
		.db 0xcc,0xcc,0xdc,0xdc,0xdc,0xec,0xec,0xec,0xfc,0xfc,0xfc,0

		libmio_thistext
		.db	6,4,"EDITOR",0

		libmio_thistext
		.db	8,4,"RUN",0

		libmio_thistext			;select default
		.db	10,4,0
		rcall	menu_text1

		libmio_thistext			;load from serial
		.db	12,4,0
		rcall	menu_text3

		libmio_thistext			;save to serial
		.db	14,4,0
		rcall	menu_text4

		libmio_thistext			;print
		.db	16,4,0
		rcall	menu_text5

		libmio_thistext
		.db	18,4,"INFO",0
		
		ldi	ZL,0			;pos 1
menu_main_1:	ldi	YL,LOW(libmio_vram+6*libmio_cols+3)
		ldi	YH,HIGH(libmio_vram+6*libmio_cols+3)
		ldi	ZH,0
menu_main_2:	ldi	XL,0x20
		cp	ZH,ZL
		brne	menu_main_3
		ldi	XL,0x16
menu_main_3:	st	Y,XL
		adiw	YL,libmio_cols*2
		inc	ZH
		cpi	ZH,menu_max
		brne	menu_main_2
		
menu_main_4:	libmio_waitkey

;------------------------------------------------------------------------------
;cursor up key
;------------------------------------------------------------------------------
menu_main_4n:	cpi	tempreg1,0xe4		;up
		brne	menu_main_5
		dec	ZL
		cpi	ZL,0xff
		brne	menu_main_1
		ldi	ZL,0x00
		rjmp	menu_main_1

;------------------------------------------------------------------------------
;cursor down key
;------------------------------------------------------------------------------
menu_main_5:	cpi	tempreg1,0xe5		;down
		brne	menu_main_6
		inc	ZL
		cpi	ZL,menu_max
		brne	menu_main_1
		ldi	ZL,menu_max-1
		rjmp	menu_main_1

;------------------------------------------------------------------------------
;enter key
;------------------------------------------------------------------------------
menu_main_6:	cpi	tempreg1,0xea		;enter
		brne	menu_main_1
		cpi	ZL,0x00			;EDIT
		brne	menu_main_7
		ldi	r16,0			;set edit text		
		rcall	menu_fsel		;file selector
		clr	ereg
		rjmp	menu_editloop

menu_main_7:	cpi	ZL,0x01			;RUN
		brne	menu_main_8
		ldi	r16,1			;set run text		
		rcall	menu_fsel		;file selector
		sts	libmio_kflags,byte0
		rcall	menu_runprog		;run programm
		cpi	ereg,0x02
		brcc	menu_editloop
menu_main_71:	libmio_waitkey
		cpi	tempreg1,0xed		;ESC		
		brne	menu_main_71
		rjmp	menu_main

menu_main_8:	cpi	ZL,0x02			;set default
		brne	menu_main_9
		ldi	r16,5			;set default-sel text		
		rcall	menu_fsel		;file selector
		mov	tempreg1,XL
		ldi	YL,LOW(1000)		;adr 1000 -> config default
		ldi	YH,HIGH(1000)		
		libeep_write
		call	mem_cpadr
		rjmp	menu_main

menu_main_9:	cpi	ZL,0x03			;load from serial
		brne	menu_main_10
		ldi	r16,3			;set receive text		
		rcall	menu_fsel		;file selector
		rcall	menu_sread
		rjmp	menu_main

menu_main_10:	cpi	ZL,0x04			;save to serial
		brne	menu_main_11
		ldi	r16,2			;set send text		
		rcall	menu_fsel		;file selector
		rjmp	menu_swrite
		
menu_main_11:	cpi	ZL,0x05			;print
		brne	menu_main_12
		ldi	r16,4			;set print text		
		rcall	menu_fsel		;file selector
		rjmp	menu_print		

menu_main_12:	rcall	menu_info		;file selector
		rjmp	menu_main		

;------------------------------------------------------------------------------
; editor loop
;------------------------------------------------------------------------------
menu_editloop:	ldi	XL,7
		sts	libmio_color,XL		;plot color = white
		lds	XL,libmio_keycode
		cpi	XL,0
		brne	menu_editloop
		call	editor_main
		sts	libmio_kflags,byte0
		lds	XL,libmio_lastkey
		cpi	XL,0xfa			;F10
		breq	menu_editloop1
		rjmp	menu_main
menu_editloop1:	call	menu_runprog		;run programm
menu_editloop2:	libmio_waitkey
		cpi	tempreg1,0xed		;ESC
		brne	menu_editloop2
		rjmp	menu_editloop

;------------------------------------------------------------------------------
; info box
;------------------------------------------------------------------------------
menu_info:	rcall	menu_ibox		;draw box
		libmio_thistext
		.db	5,5,0
		rcall	menu_itext1		;show version
		libmio_thistext
		.db	7,2,"(c) 2006/7 Joerg Wolfram",0
		libmio_thistext
		.db	8,2,"This software is licensed",0
		libmio_thistext
		.db	9,2,"under the terms of the",0
		libmio_thistext
		.db	10,2,"GPL.",0
		libmio_waitkey
		ret

;------------------------------------------------------------------------------
; the yellow box
;------------------------------------------------------------------------------
menu_ibox:	ldi	XL,2
		ldi	XH,4
menu_ibox_0:	libmio_gotoxy			;set pos				
		ldi	ZL,24
		ldi	tempreg1,' '
		libmio_mtext
		inc	XH
		cpi	XH,13
		brne	menu_ibox_0
		ldi	YL,6
		sts	libmio_color,YL
		ldi	YL,3
		ldi	YH,8
		ldi	ZL,55
		ldi	ZH,23
		libmio_box
		ret

;------------------------------------------------------------------------------
; file selector
;------------------------------------------------------------------------------
menu_fsel:	rcall	menu_ibox
		libmio_thistext
		.db	5,3,"SELECT FILE TO ",0
		cpi	r16,0			;edit file
		brne	menu_fsel1
		libmio_thistext
		.db	255,"EDIT",0
		rjmp	menu_fsel9
menu_fsel1:	cpi	r16,1			;run file
		brne	menu_fsel2
		libmio_thistext
		.db	255,"RUN",0
		rjmp	menu_fsel9
menu_fsel2:	cpi	r16,2			;send file
		brne	menu_fsel3
		libmio_thistext
		.db	255,"SEND",0
		rjmp	menu_fsel9
menu_fsel3:	cpi	r16,3			;receive file
		brne	menu_fsel4
		libmio_thistext
		.db	255,"RECEIVE",0
		rjmp	menu_fsel9
menu_fsel4:	cpi	r16,4			;print file
		brne	menu_fsel5
		libmio_thistext
		.db	255,"PRINT",0
		rjmp	menu_fsel9
menu_fsel5:	libmio_thistext
		.db	255,"DEFAULT",0


menu_fsel9:	rcall	menu_flist
		ldi	tempreg4,bas_maxprog
		rcall	menu_msel
		cpi	XL,0xff			;no select
		brne	menu_fsel10
		pop	tempreg1
		pop	tempreg1
		rjmp	menu_main
menu_fsel10:	sts	libmio_ram+46,ZL
		mov	tempreg1,ZL
		jmp	mem_setprog		;set program pointers
		
;------------------------------------------------------------------------------
; show flash filelist
;------------------------------------------------------------------------------
menu_flist:	ldi	YL,LOW(1000)		;adr 1000 -> config default
		ldi	YH,HIGH(1000)		
		libeep_read			;default program
		mov	tempreg2,tempreg1	;copy
		clr	ctrl			;first program number
menu_flist1:	ldi	YL,LOW(libmio_vram+7*libmio_cols+5)		
		ldi	YH,HIGH(libmio_vram+7*libmio_cols+5)		
		ldi	XH,libmio_cols
		mul	XH,ctrl
		add	YL,r0
		adc	YH,r1
		cp	tempreg2,ctrl
		breq	menu_flist2
		ldi	tempreg1,0x20		;space
		rjmp	menu_flist3
menu_flist2:	ldi	tempreg1,0x1d		;ball
menu_flist3:	st	Y+,tempreg1		
		call	mem_readnamec
		inc	ctrl
		cpi	ctrl,bas_maxprog
		brne	menu_flist1
		ret

;------------------------------------------------------------------------------
; read from serial
;------------------------------------------------------------------------------
menu_sread:	libmio_clrscr
		libmio_emode
		libmio_thistext			;show program name
		.db	0,0,"RECEIVING...",0
		sts	libmio_edstart,byte0
		call	lce_pnew
		clr	ctrl			;start with line 0
		ldi	ZL,LOW(libmio_vram+libmio_cols)
		ldi	ZH,HIGH(libmio_vram+libmio_cols)
		mov	r14,ZL
		mov	r15,ZH	
		ldi	tempreg1,0x11		;XON
		libmio_pser			;send
		rjmp	menu_sreadv
menu_sread1:	ldi	tempreg2,libmio_ecols
		mov	ZL,r14
		mov	ZH,r15
		adiw	ZL,libmio_ecols
		mov	r14,ZL
		mov	r15,ZH	
menu_sreadv:	libmio_gser
		cpi	tempreg1,0x20		;ignore all less than space
		brcs	menu_sreadv
		cpi	tempreg1,'#'		;end
		breq	menu_sread5		
		rjmp	menu_sread2a
menu_sread2:	libmio_gser			;get serial
		cpi	tempreg1,0x20		;space
		brcs	menu_sread3		;is less -> control
menu_sread2a:	st	Z+,tempreg1
		dec	tempreg2
		brne	menu_sread2		;OK
menu_sread3:	inc	ctrl
		cpi	ctrl,0x01
		brne	menu_sread4
		ldi	ZL,LOW(libmio_vram+3*libmio_cols)
		ldi	ZH,HIGH(libmio_vram+3*libmio_cols)
		mov	r14,ZL
		mov	r15,ZH	
		rjmp	menu_sreadv
menu_sread4:	cpi	ctrl,bas_lines+1
		brne	menu_sread1					    
menu_sread5:	libmio_thistext			;show program name
		.db	0,0,"COMPRESSING...",0
		call	lce_rpsave
		libmio_nmode
		ret

;------------------------------------------------------------------------------
; write to serial
;------------------------------------------------------------------------------
menu_swrite:	libmio_thistext			;show program name
		.db	21,1,"SENDING PROGRAM...",0
		ldi	XL,0x01			;serial
		sts	libmio_channel,XL	;set channel
		rcall	menu_outname
		call	lce_findlast
		ldi	tempreg1,'#'		;stopper
		libmio_outchar
		ldi	tempreg1,0x0d		;carriage return
		libmio_outchar
		ldi	tempreg1,0x0a		;line feed
		libmio_outchar
		rjmp	menu_main		;back to main loop

;------------------------------------------------------------------------------
; print program
;------------------------------------------------------------------------------
menu_print:	libmio_thistext			;show program name
		.db	21,1,"PRINTING...",0
		ldi	XL,0x02			;printer
		sts	libmio_channel,XL	;set channel
		libmio_newline
		libmio_newline
		rcall	menu_outname
		libmio_newline
		call	lce_findlast
		ldi	tempreg1,0x0c		;form feed
		libmio_outchar
		rjmp	menu_main		;back to main loop

;------------------------------------------------------------------------------
; output program name
;------------------------------------------------------------------------------
menu_outname:	libmio_thistext
		.db	255,"PROGRAM: ",0
		ldi	YL,LOW(bas_inbuf)
		ldi	YH,HIGH(bas_inbuf)
		call	mem_readname
		st	Y+,byte0		;eol
		ldi	ZL,LOW(bas_inbuf)
		ldi	ZH,HIGH(bas_inbuf)
		libmio_ramtext		
		libmio_newline
		ret

;------------------------------------------------------------------------------
; select a program from list
;------------------------------------------------------------------------------
menu_msel:	ldi	YL,LOW(1000)		;adr 1000 -> config default
		ldi	YH,HIGH(1000)		
		libeep_read
		mov	ZL,tempreg1		;default pos
menu_msel_1:	ldi	YL,LOW(libmio_vram+7*libmio_cols+4)
		ldi	YH,HIGH(libmio_vram+7*libmio_cols+4)
		ldi	ZH,0
menu_msel_2:	ldi	XL,0x20
		cp	ZH,ZL
		brne	menu_msel_3
		ldi	XL,0x16
menu_msel_3:	st	Y,XL
		adiw	YL,libmio_cols
		inc	ZH
		cp	ZH,tempreg4
		brne	menu_msel_2
		
menu_msel_4:	libmio_waitkey			;wait for keypress

;------------------------------------------------------------------------------
;cursor up key
;------------------------------------------------------------------------------
		cpi	tempreg1,0xe4		;up
		brne	menu_msel_5
		dec	ZL
		cpi	ZL,0xff
		brne	menu_msel_1
		ldi	ZL,0x00
		rjmp	menu_msel_1

;------------------------------------------------------------------------------
;cursor down key
;------------------------------------------------------------------------------
menu_msel_5:	cpi	tempreg1,0xe5		;down
		brne	menu_msel_6
		inc	ZL
		cp	ZL,tempreg4
		brne	menu_msel_1
		mov	ZL,tempreg4
		dec	ZL
		rjmp	menu_msel_1

;------------------------------------------------------------------------------
;enter key
;------------------------------------------------------------------------------
menu_msel_6:	cpi	tempreg1,0xea		;enter
		brne	menu_msel_7
		mov	XL,ZL
		ret
		
;------------------------------------------------------------------------------
;escape key
;------------------------------------------------------------------------------
menu_msel_7:	cpi	tempreg1,0xed		;esc
		brne	menu_msel_1
		ldi	XL,0xff
		ret		
		

;------------------------------------------------------------------------------
;set default program
;------------------------------------------------------------------------------
menu_text1:	libmio_thistext
		.db	255,"SELECT DEFAULT",0
		ret

;------------------------------------------------------------------------------
;text save to memory card
;------------------------------------------------------------------------------
menu_text2:	libmio_thistext
		.db	255,"FILE FUNCTIONS",0
		ret	

;------------------------------------------------------------------------------
;text save to memory card
;------------------------------------------------------------------------------
menu_text6:	libmio_thistext
		.db	255,"FUNCTION N/A ",0
		ret

;------------------------------------------------------------------------------
;text load from serial
;------------------------------------------------------------------------------
menu_text3:	libmio_thistext
		.db	255,"RECEIVE FILE",0
		ret

;------------------------------------------------------------------------------
;text save to serial
;------------------------------------------------------------------------------
menu_text4:	libmio_thistext
		.db	255,"SEND FILE",0
		ret

;------------------------------------------------------------------------------
;text print listing
;------------------------------------------------------------------------------
menu_text5:	libmio_thistext
		.db	255,"PRINT LISTING",0
		ret
		
;------------------------------------------------------------------------------
;run program
;------------------------------------------------------------------------------
menu_runprog:	libmio_clrscr			;clear screen from menu			
		call	tbrun			;run programm
		libmio_nmode			;normal mode
		sts	libmio_channel,byte0	;set output to screen
		libmio_newline
		libmio_thistext
		.db libmio_rows-1,0,0x9f,0x17,0x18,0x19,0x1a,0x1a,0x9f,0x19,0x1a,0x1b,0x9f,0
		ret	

;------------------------------------------------------------------------------
;clear screen and draw box
;------------------------------------------------------------------------------
menu_maxbox:	libmio_clrscr			;clear screen
		ldi	YL,5
		sts	libmio_color,YL
		ldi	YL,0
		ldi	YH,0
		ldi	ZL,59
		ldi	ZH,45
		libmio_box		
		ret

;------------------------------------------------------------------------------
;info text (version)
;------------------------------------------------------------------------------
menu_itext1:	libmio_thistext
		.db 255,"AVR CHIPBASIC V0.91",0
		ret	
		